//
//  Weibull3.cpp
//  
//
//  Created by Arne Pommerening on 06/09/2016.
//
//

#include <stdio.h>
#include <Rcpp.h>
#include <cmath>
#include <vector>
using namespace Rcpp;
using namespace std;
long idum = -5;


/**
 * Simulates a Weibull distributed random number.
 */
double rweibull3(double p, double alpha, double beta, double gamma) {
    return alpha + beta * pow(-log(1 - p), 1.0 / gamma);
}

#define MBIG 1000000000
#define MSEED 161803398
#define MZ 0
#define FAC (1.0/MBIG)

/*
 * (C) Random number generator. Copr. 1986-92 Numerical Recipes Software.
 */
double ran3() {
    static int inext, inextp;
    static long ma[56];
    static int iff = 0;
    long mj, mk;
    int i, ii, k;
    
    if (idum < 0 || iff == 0) {
        iff = 1;
        mj = MSEED - (idum < 0 ? -idum : idum);
        mj %= MBIG;
        ma[55] = mj;
        mk = 1;
        for (i = 1; i <= 54; i++) {
            ii = (21 * i) % 55;
            ma[ii] = mk;
            mk = mj - mk;
            if (mk < MZ)
                mk += MBIG;
            mj = ma[ii];
        }
        for (k = 1;k <= 4; k++)
            for (i = 1; i <= 55; i++) {
                ma[i] -= ma[1+(i+30) % 55];
                if (ma[i] < MZ) ma[i] += MBIG;
            }
        inext = 0;
        inextp = 31;
        idum = 1;
    }
    if (++inext == 56)
        inext = 1;
    if (++inextp == 56)
        inextp = 1;
    mj = ma[inext] - ma[inextp];
    if (mj < MZ)
        mj += MBIG;
    ma[inext] = mj;
    return mj * FAC;
}
#undef MBIG
#undef MSEED
#undef MZ
#undef FAC


/**
 * Simulates Weibull distributed random numbers (tree stem diameters) up to a certain basal area.
 */
// [[Rcpp::export]]
NumericVector simulateDiameterList(double alpha, double beta, double gamma, double ba) {
    const double pi  = 3.141592653589793238463;
    vector<double> xdbh;
    double baSim = 0;
    while(baSim < ba){
        double p = ran3();
        double d = rweibull3(p, alpha, beta, gamma);
        xdbh.push_back(d);
        baSim += pi * pow(d / 200, 2);
    }
    return wrap(xdbh);
}
