//
//  Weibull3.cpp
//  
//
//  Created by Arne Pommerening on 06/09/2016. Updated on 11/06/2017.
//
//

#include <Rcpp.h>
#include <cmath>
#include <vector>
#include <random>

// [[Rcpp::plugins("cpp11")]]

using namespace Rcpp;
using namespace std;

int seed = 1;
static std::default_random_engine gen(seed);
static std::uniform_real_distribution<double> u(0, 1);

/**
 * Simulates a Weibull distributed random number.
 */
double rweibull3(double p, double alpha, double beta, double gamma) {
    return alpha + beta * pow(-log(1 - p), 1.0 / gamma);
}


/**
 * Simulates Weibull distributed random numbers (tree stem diameters) up to a certain basal area.
 */
// [[Rcpp::export]]
NumericVector simulateDiameterList(double alpha, double beta, double gamma, double ba) {
    const double pi  = 3.141592653589793238463;
    vector<double> xdbh;
    double baSim = 0;
    while(baSim < ba){
        // double p = ran3();
        double p = u(gen);
        double d = rweibull3(p, alpha, beta, gamma);
        xdbh.push_back(d);
        baSim += pi * pow(d / 200, 2);
    }
    return wrap(xdbh);
}
