# Example 1b, Ap - 06.09.2016
# Estimation of parameters of Weibull 3 using Rcpp
# Robinson and Hamann (2011, p. 164); Clutter et al. (1983, p. 127)

rm(list = ls())
options(digits = 6, width = 200)


library(Rcpp)

# xfile <- "Z:/Dysgu/ResearchSchool/ScientificProgrammingSimulation/Simulation/R/"
xfile <- "/Users/arng0001/Dropbox/Gwaith/ScientificProgrammingSimulation/Simulation/R/"

sourceCpp(paste(xfile, "Weibull3.cpp", sep = ""))

# Load data
# example1b <- read.table("Z:/Dysgu/ResearchSchool/ScientificProgrammingSimulation/Simulation/R/Example1b.txt", header=T)							 
example1b <- read.table("/Users/arng0001/Dropbox/Gwaith/ScientificProgrammingSimulation/Simulation/R/Example1b.txt", header=T)							 

dim(example1b)																
names(example1b)																


# Fit Weibull distribution to observed data using ML
dweibull3 <- function(x, gamma, beta, alpha) {
  (gamma/beta) * ((x -alpha)/beta)^(gamma - 1) * (exp(-((x - alpha)/beta)^gamma))
}

ll.w3 <- function(p, data)
  sum(log(dweibull3(data, p[1], p[2], p[3])))

mle.w3.nm <- optim(c(gamma = 1, beta = 5, alpha = 10),
                   ll.w3, data = example1b$dbh2002, hessian = TRUE, control = list(fnscale = -1)) # Nelder-Mead method

mle.w3.nm$par # model parameters

mle.w3.nm$value # log-likelihood

sqrt(diag(solve(-mle.w3.nm$hessian))) # asymptotic estimates of the standard errors

# Check whether the curve looks ok
xx <- seq(10, 60, 1)
hist(example1b$dbh2002, freq = F, breaks = 50, xlim = c(10, 60))
lines(xx, dweibull3(xx, mle.w3.nm$par[1], mle.w3.nm$par[2], mle.w3.nm$par[3]), lty = 1, col = "red")

# Calculate basal area, number of trees and dg of orginial tree list
example1b$ba <- pi * (example1b$dbh2002 / 200)^2
ba <- sum(example1b$ba) 
ba
length(example1b$dbh2002)
sqrt(sum(example1b$dbh2002^2)/length(example1b$dbh2002))
sqrt(40000 / pi * ba / length(example1b$dbh2002)) # check


dbh <- simulateDiameterList(mle.w3.nm$par[3], mle.w3.nm$par[2], mle.w3.nm$par[1], ba)

# Calculate basal area, number of trees and dg of simulated tree list
ba.sim <- sum(pi * (dbh / 200)^2)
ba.sim
length(dbh)
sqrt(sum(dbh^2)/length(dbh))
sqrt(40000 / pi * ba.sim / length(dbh)) # check

# Check whether the curve looks ok
xx <- seq(10, 60, 1)
hist(dbh, freq = F, breaks = 50, xlim = c(10, 60))
lines(xx, dweibull3(xx, mle.w3.nm$par[1], mle.w3.nm$par[2], mle.w3.nm$par[3]), lty = 1, col = "red")
